package org.claros.commons.mail.parser;

import java.io.ByteArrayOutputStream;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Enumeration;

import javax.mail.Message;
import javax.mail.MessagingException;
import javax.mail.Multipart;
import javax.mail.Part;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.claros.commons.mail.models.Email;
import org.claros.commons.mail.models.EmailPart;
import org.claros.commons.mail.models.EmailHeader;
import org.claros.commons.mail.utility.Utility;
import org.claros.commons.utility.Formatter;

/**
 * @author Umut Gokbayrak
 *
 */
public class MessageParser {
    private static Log log = LogFactory.getLog(MessageParser.class);
    
    /**
     * 
     */
    public MessageParser() {
        super();
    }

	public static final Email parseMessage(Message pop3Msg) {
		Email msg = new Email();

		ArrayList parts = new ArrayList();
		parts = fetchParts(pop3Msg, parts);
		if (parts != null) {
			EmailPart part = null;
			for (int i=0;i<parts.size();i++) {
				part = (EmailPart)parts.get(i);
				part.setId(i);
			}
		}
		msg.setParts(parts);
    	
    	// get base headers
		try {
            EmailHeader header = new EmailHeader();
            header.setFrom(pop3Msg.getFrom());
            header.setTo(pop3Msg.getRecipients(Message.RecipientType.TO));
            header.setCc(pop3Msg.getRecipients(Message.RecipientType.CC));
            header.setBcc(pop3Msg.getRecipients(Message.RecipientType.BCC));
            header.setReplyTo(pop3Msg.getReplyTo());
            header.setDate(pop3Msg.getSentDate());
            header.setSize(pop3Msg.getSize());
            header.setSubject(pop3Msg.getSubject());

			// now set the human readables.
			header.setDateShown(Formatter.formatDate(header.getDate(), "dd.MM.yyyy HH:mm"));
			header.setFromShown(Utility.addressArrToString(header.getFrom()));
			header.setToShown(Utility.addressArrToString(header.getTo()));
			header.setCcShown(Utility.addressArrToString(header.getCc()));
			header.setSizeShown(Utility.sizeToHumanReadable(header.getSize()));

            msg.setBaseHeader(header);
        } catch (Exception e) {
            log.error("Exception occured while parsing the message base headers", e);
        }

		// store all headers
		try {
            Enumeration enum = pop3Msg.getAllHeaders();
            String name, val = "";
            Object tmp = null;
            while (enum.hasMoreElements()) {
            	tmp = enum.nextElement();
            	name = (tmp == null) ? "" : tmp.toString();
            	tmp = pop3Msg.getHeader(name);
            	val = (tmp == null) ? "" : tmp.toString();
            	msg.addHeader(name, val);
            }
        } catch (MessagingException e1) {
            log.error("Exception occured while parsing the message generic all headers", e1);
        }
		return msg;
	}

	/**
	 * A recursive algorithm travelling through a MIME message, looking
	 * at the mime types of each part and decodes it into a text content.
	 * @param p
	 * @param parts
	 * @return ArrayList of EmailParts
	 */
	private static ArrayList fetchParts(Part p, ArrayList parts) {
		if (p == null) return null;

		try {
            if (!p.isMimeType("text/rfc822-headers") && p.isMimeType("text/*")) {
            	try {
                    EmailPart myPart = new EmailPart();
                    myPart.setSize(p.getSize());
                    myPart.setContentType(p.getContentType());
                    myPart.setFileName(p.getFileName());
                    myPart.setDisposition(p.getDisposition());
                    Object pContent = p.getContent();
                    if (pContent != null) {
                    	myPart.setContent(pContent.toString());
                    } else {
                    	myPart.setContent("Illegal content");
                    }
                    parts.add(myPart);
                } catch (Exception e) {
                    log.error("Part is mimeType text/rfc822-headers and is mimeType text/* but exception occured", e);
                }
            } else if (p.isMimeType("multipart/*")) {
                try {
                    Multipart mp = (Multipart) p.getContent();
                    int count = mp.getCount();
                    for (int i = 0; i < count; i++) {
                    	fetchParts(mp.getBodyPart(i), parts);
                    }
                } catch (Exception e) {
                    log.error("Part is mimeType multipart/* but exception occured", e);
                }
		} else if (p.isMimeType("message/rfc822")) {
            	fetchParts((Part) p.getContent(), parts);
            } else {
            	try {
                    EmailPart myPart = new EmailPart();
                    myPart.setSize(p.getSize());
                    myPart.setContentType(p.getContentType());
                    myPart.setFileName((p.getFileName() == null) ? "rfc822.txt" : p.getFileName());
                    myPart.setDisposition(p.getDisposition());
                    String headContentID[] = p.getHeader("Content-ID");
                    if (headContentID != null) {
                    	myPart.setContentId(headContentID[0]);
                    }
                    InputStream is = p.getInputStream();
                    ByteArrayOutputStream baos = new ByteArrayOutputStream();
                    int c;
                    while ((c = is.read()) != -1) {
                    	baos.write(c);
                    }
                    myPart.setContent(baos);
                    parts.add(myPart);
                } catch (Exception e) {
                    log.error("An exception occured while parsing this part.", e);
                }
            }
        } catch (Exception e) {
            log.error("An exception occured while parsing the parts of the message.", e);
        }
		return parts;
	}

}
