package org.claros.groupware.calendar.utility;

import java.sql.Timestamp;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;

import org.claros.groupware.calendar.models.CalendarItemWrap;

/**
 * @author Umut Gokbayrak
 */
public class Utility {
	/**
	 * Input olarak gelen tarihi parametreye gre ait olduu gnn bana veya
	 * sonuna gnderir. rnein  haftann ilk gnn bulduumuzda calendar objemizi 
	 * o gnn ilk salisesi 00:00.000 saatine evirir. Bunun iin isBegin parametresi
	 * true olmaldr.  
	 * @param cal
	 * @param isBegin
	 * @return
	 */
	public static Timestamp getLimitTimestampFromCalendar(Calendar cal, boolean isBegin) {
		SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss.SSS");
		SimpleDateFormat sdfBegin = new SimpleDateFormat("yyyy-MM-dd 00:00:00.000");
		SimpleDateFormat sdfEnd = new SimpleDateFormat("yyyy-MM-dd 23:59:59.999");

		Calendar calTmp = Calendar.getInstance();
		if (isBegin) {
			Timestamp begin = Timestamp.valueOf(sdfBegin.format(cal.getTime()));
			calTmp.setTime(new Date(begin.getTime()));
			calTmp.add(Calendar.MILLISECOND, -1);
		} else {
			Timestamp end = Timestamp.valueOf(sdfEnd.format(cal.getTime()));
			calTmp.setTime(new Date(end.getTime()));
			calTmp.add(Calendar.MILLISECOND, 1);
		}
		return Timestamp.valueOf(sdf.format(calTmp.getTime()));
	}

	public static ArrayList getBeginEndTimestamps(Calendar cal, int timeVar) {
		Timestamp begin = null;
		Timestamp end = null;
		cal.setFirstDayOfWeek(Calendar.MONDAY);
		
		switch (timeVar) {
			case Constants.DAY : 
				begin = getLimitTimestampFromCalendar(cal, true);
				end = getLimitTimestampFromCalendar(cal, false);
				break;
			case Constants.WEEK :
				// Eer buraya gelmi olan gn pazartesi deilse pazartesi yapmalyz.
				int dayOfWeek = cal.get(Calendar.DAY_OF_WEEK);
				while (dayOfWeek != Calendar.MONDAY) {
					cal.add(Calendar.DATE, -1);
					dayOfWeek = cal.get(Calendar.DAY_OF_WEEK);
				}
				
				Calendar sunday = (Calendar)cal.clone();
				sunday.add(Calendar.DATE, 6);
				begin = getLimitTimestampFromCalendar(cal, true);
				end = getLimitTimestampFromCalendar(sunday, false);
				sunday = null;
				break;
			case Constants.MONTH : 
				cal.set(Calendar.DAY_OF_MONTH, 1);

				Calendar lastDay = (Calendar)cal.clone();
				int l = lastDay.getActualMaximum(Calendar.DAY_OF_MONTH);
				lastDay.set(Calendar.DAY_OF_MONTH, l);

				begin = getLimitTimestampFromCalendar(cal, true);
				end = getLimitTimestampFromCalendar(lastDay, false);
				lastDay = null;
				break;
		}
		
		ArrayList out = new ArrayList();
		out.add(begin);
		out.add(end);
		return out;
	}

	/**
	 * Tekrarlanan olaylarn belirtilen zaman aralnda (begin -> end) gerekleip 
	 * gereklemediini anlamak iin o ajanda kayd belirtilen kriter ile 
	 * end tarihinden kk olduu srece ileriye doru sarmak gereklidir. Bu 
	 * metod bu ilemi yapar.  
	 * @param tmpApp			ajanda kayd
	 * @param timeCriteria		ileriye doru sarlan kriter. gun/hafta/ay/yl vs...
	 * @param ascend			kaar kaar arttrlmal roll ederken. 
	 * @param begin				hangi zaman aralna kadar saryoruz. balang parametresi
	 * @param end				hangi zaman aralna kadar saryoruz. biti parametresi
	 * @return					sarlm halde ajanda kaydn dndrr
	 */
	public static CalendarItemWrap rollAppointment(CalendarItemWrap tmpApp, int timeCriteria, int ascend, Timestamp begin, Timestamp end) {
		Timestamp tmpDate = tmpApp.getRecordDate();

		while (tmpDate.before(begin)) {
			Calendar tmpCal = Calendar.getInstance();
			tmpCal.setTime(new Date(tmpDate.getTime()));
			tmpCal.add(timeCriteria, ascend);
			tmpDate = new Timestamp(tmpCal.getTime().getTime());
		}
		if (tmpDate.before(end)) {
			// tekrarlanan olaylarda olayn tekrarland ve o hafta/ay/yl iinde yer ald tarihi burada buluyoruz.
			tmpApp.setOccuringDate(tmpDate);
			return tmpApp;
		}
		return null;
	}

	public static CalendarItemWrap rollAppointmentOnce(CalendarItemWrap tmpApp, int timeCriteria, int ascend, Timestamp end) {
		Timestamp tmpDate = tmpApp.getOccuringDate();

		Calendar tmpCal = Calendar.getInstance();
		tmpCal.setTime(new Date(tmpDate.getTime()));
		tmpCal.add(timeCriteria, ascend);
		tmpDate = new Timestamp(tmpCal.getTime().getTime());

		if (tmpDate.before(end)) {
			tmpApp.setOccuringDate(tmpDate);
			return tmpApp;
		}
		return null;
	}

	/**
	 * @param cal
	 * @param b
	 * @return
	 */
	public static Timestamp getHourLimitTimestampFromCalendar(Calendar cal, boolean isBegin) {
		SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss.SSS");
		SimpleDateFormat sdfBegin = new SimpleDateFormat("yyyy-MM-dd HH:00:00.000");
		SimpleDateFormat sdfEnd = new SimpleDateFormat("yyyy-MM-dd HH:59:59.999");

		Calendar calTmp = Calendar.getInstance();
		if (isBegin) {
			Timestamp begin = Timestamp.valueOf(sdfBegin.format(cal.getTime()));
			calTmp.setTime(new Date(begin.getTime()));
			calTmp.add(Calendar.MILLISECOND, -1);
		} else {
			Timestamp end = Timestamp.valueOf(sdfEnd.format(cal.getTime()));
			calTmp.setTime(new Date(end.getTime()));
			calTmp.add(Calendar.MILLISECOND, 1);
		}
		return Timestamp.valueOf(sdf.format(calTmp.getTime()));
	}
}
