package org.claros.groupware.contacts.actions;

import java.util.ArrayList;
import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.util.LabelValueBean;

import org.claros.commons.models.AuthProfile;
import org.claros.groupware.common.actions.BaseLoggedAction;
import org.claros.groupware.contacts.controllers.ContactsController;
import org.claros.groupware.contacts.controllers.GroupsController;
import org.claros.groupware.contacts.models.Contact;
import org.claros.groupware.contacts.models.ContactGroup;
import org.claros.groupware.contacts.models.ContactGroupItem;
import org.claros.groupware.contacts.models.GroupFormBean;
import org.claros.groupware.preferences.controllers.UserPrefsController;
import org.claros.groupware.preferences.utility.UserPrefConstants;

/**
 * @version 	1.0
 * @author		Umut Gkbayrak
 */
public class ShowGroupAction extends BaseLoggedAction {

	public ActionForward myExecute(ActionMapping mapping, ActionForm form, HttpServletRequest request, HttpServletResponse response) throws Exception {
		AuthProfile auth = getAuthProfile(request);
		
		Long id = new Long((String)getVariable(request, "id"));
		
		// first the members and all the contacts first
		List members = GroupsController.getGroupItemsByGroup(auth, id);
		List contacts = ContactsController.getContactsByNamePrefix(auth, "ALL");

		// remove the already selected members from the contacts list
		ContactGroupItem item = null;
		Long memberId = null;
		Contact cnt = null;
		for (int i=0; i<members.size();i++) {
			item = (ContactGroupItem)members.get(i);
			memberId = item.getContactId();
			cnt = findInContacts(contacts, memberId);
			if (cnt != null) {
				contacts.remove(cnt);
			}
		}

		// determine how to display the names
		String displayType = UserPrefsController.getUserSetting(auth, UserPrefConstants.displayType);
		boolean nameFirst = true;
		if (displayType != null && displayType.equals(UserPrefConstants.DISPLAY_TYPE_SURNAME_FIRST)) {
			nameFirst = false;
		}

		// prepare the contacts multiselect list
		ArrayList contactsCmb = new ArrayList();
		if (contacts != null) {
			Contact tmp = null;

			String fullName = null;
			for (int i=0; i<contacts.size(); i++) {
				tmp = (Contact)contacts.get(i);
				fullName = getFullName(tmp, nameFirst);
				if (fullName != null) {
					contactsCmb.add(new LabelValueBean(fullName, tmp.getId().toString()));
				}
			}
		}
		request.setAttribute("contacts", contactsCmb);

		
		// fetch the group details
		ContactGroup group = GroupsController.getGroupById(auth, id);
		GroupFormBean bean = new GroupFormBean();
		bean.setGroupName(group.getShortName());
		bean.setId(id.toString());
		request.setAttribute("GroupFormBean", bean);
		
		
		// prepare the members multiselect list
		ArrayList membersCmb = new ArrayList();
		if (members != null) {
			ContactGroupItem tmp = null;

			String fullName = null;
			Contact c = null;
			for (int i=0; i<members.size(); i++) {
				tmp = (ContactGroupItem)members.get(i);
				c = ContactsController.getContactById(auth, tmp.getContactId());
				fullName = getFullName(c, nameFirst);

				if (fullName != null) {
					membersCmb.add(new LabelValueBean(fullName, c.getId().toString()));
				}
			}
		}
		request.setAttribute("members", membersCmb);
		
		return mapping.findForward("/group.jsp");
	}

	/**
	 * 
	 * @param contacts
	 * @param id
	 * @return
	 */
	private Contact findInContacts(List contacts, Long id) {
		Contact cnt = null;
		for (int j=0; j<contacts.size(); j++) {
			cnt = (Contact)contacts.get(j);
			if (cnt.getId().equals(id)) {
				return cnt;
			}
		}
		return null;
	}

	/**
	 * 
	 * @param contact
	 * @param nameFirst
	 * @return
	 */
	private String getFullName(Contact contact, boolean nameFirst) {
		String firstName = contact.getFirstName();
		String middleName = contact.getMiddleName();
		String lastName = contact.getLastName();
		String fullName = "";	
		if (nameFirst) {
			fullName = firstName + " " + lastName;
			if (middleName != null && !middleName.trim().equals("")) {
				fullName = firstName + " " + middleName + " " + lastName;
			}
		} else {
			fullName = lastName + ", " + firstName;
			if (middleName != null && !middleName.trim().equals("")) {
				fullName = lastName + ", " + firstName + " " + middleName;
			}
		}
		return fullName;
	}
}
