package org.claros.groupware.contacts.controllers;

import java.util.List;
import java.util.Locale;

import org.claros.commons.exception.NoPermissionException;
import org.claros.commons.models.AuthProfile;
import org.claros.groupware.common.utility.Constants;
import org.claros.groupware.common.utility.Utility;
import org.claros.groupware.contacts.models.Contact;
import org.claros.groupware.preferences.controllers.UserPrefsController;
import org.claros.groupware.preferences.utility.UserPrefConstants;

import com.jenkov.mrpersister.impl.mapping.AutoGeneratedColumnsMapper;
import com.jenkov.mrpersister.itf.IGenericDao;
import com.jenkov.mrpersister.itf.mapping.IObjectMappingKey;
import com.jenkov.mrpersister.util.JdbcUtil;

/**
 * @author Umut Gokbayrak
 */
public class ContactsController {
	
	public static List getContactsByNamePrefix(AuthProfile auth, String prefix) throws Exception {
		IGenericDao dao = null;
		List contacts = null;
		try {
			dao = Utility.getDbConnection();
			String username = auth.getUsername();
			
			String displayType = UserPrefsController.getUserSetting(auth, UserPrefConstants.displayType);
			String sql = null;
			if (displayType == null) {
				displayType = UserPrefConstants.DISPLAY_TYPE_NAME_FIRST;
			}
			
			if (prefix.equals("ALL")) {
				prefix = "";
			}
			if (displayType == null || displayType.equals(UserPrefConstants.DISPLAY_TYPE_NAME_FIRST)) {
				sql = "select * from CONTACTS where user=? and upper(FIRST_NAME) like '" + prefix + "%'";
			} else if (displayType.equals(UserPrefConstants.DISPLAY_TYPE_SURNAME_FIRST)) {
				sql = "select * from CONTACTS where user=? and upper(LAST_NAME) like '" + prefix + "%'";
			}
			contacts = dao.readList(Contact.class, sql, new Object[] {username});
			contacts = org.claros.groupware.contacts.utility.Utility.sortContacts(contacts, displayType.equals(UserPrefConstants.DISPLAY_TYPE_NAME_FIRST));
		} finally {
			JdbcUtil.close(dao);
			dao = null;
		}
		return contacts;
	}

	/**
	 * @param auth
	 * @param contact
	 */
	public static void saveContact(AuthProfile auth, Contact contact) throws Exception {
		IGenericDao dao = null;
		try {
			dao = Utility.getDbConnection();
			
			Long id = contact.getId();
			if (id == null) {
				// it is an insert
				IObjectMappingKey myObj = Constants.persistMan.getObjectMappingFactory().createInstance(Contact.class, new AutoGeneratedColumnsMapper(true));
				dao.insert(myObj, contact);
			} else {
				// it is an update
				Contact tmp = getContactById(auth, contact.getId());
				if (!tmp.getUser().equals(auth.getUsername())) {
					throw new NoPermissionException();
				}
				dao.update(contact);
			}
		} finally {
			JdbcUtil.close(dao);
			dao = null;
		}
	}

	public static Contact getContactById(AuthProfile auth, Long id) throws Exception {
		IGenericDao dao = null;
		Contact result = null;
		try {
			dao = Utility.getDbConnection();
			result = (Contact)dao.readByPrimaryKey(Contact.class, id);

			if (!result.getUser().equals(auth.getUsername())) {
				throw new NoPermissionException();
			}
		} finally {
			JdbcUtil.close(dao);
			dao = null;
		}
		return result;
	}

	/**
	 * @param id
	 */
	public static void deleteContact(AuthProfile auth, Long id) throws Exception {
		Contact tmp = getContactById(auth, id);
		if (!tmp.getUser().equals(auth.getUsername())) {
			throw new NoPermissionException();
		}

		IGenericDao dao = null;
		try {
			dao = Utility.getDbConnection();
			dao.deleteByPrimaryKey(Contact.class, id);
		} catch (Exception e) {
			// do nothing sier
		} finally {
			JdbcUtil.close(dao);
			dao = null;
		}
	}

	/**
	 * @param auth
	 * @param address
	 * @param company
	 * @param email
	 * @param name
	 * @param phone
	 * @return
	 */
	public static List searchContacts(AuthProfile auth, String address, String company, String email, String name, String phone) throws Exception {
		IGenericDao dao = null;
		List contacts = null;
		try {
			dao = Utility.getDbConnection();
			String username = auth.getUsername();
			
			String sql = "select * from CONTACTS where user=? ";
			
			// address conditions
			if (address != null && address.trim().length() != 0) {
				sql += "and (" +
					   "	upper(HOME_ADDRESS) like '%" + address + "%' or " +
					   "	upper(HOME_CITY) like '%" + address + "%' or " +
					   "	upper(HOME_PROVINCE) like '%" + address + "%' or " +
					   "	HOME_ZIP like '%" + address + "%' or " +
					   "	upper(HOME_COUNTRY) like '%" + address + "%' or " +
					   "	upper(WORK_ADDRESS) like '%" + address + "%' or " +
					   "	upper(WORK_CITY) like '%" + address + "%' or " +
					   "	upper(WORK_PROVINCE) like '%" + address + "%' or " +
					   "	WORK_ZIP like '%" + address + "%' or " +
					   "	upper(WORK_COUNTRY) like '%" + address + "%') ";
			}
			
			// company criteria
			if (company != null && company.trim().length() != 0) {
				sql += "or upper(WORK_COMPANY) like '%" + company + "%' ";
			}

			// search emails
			if (email != null && email.trim().length() != 0) {
				sql += "or (" +
					   "	upper(EMAIL_PRIMARY) like '%" + email + "%' or " +
					   "	upper(EMAIL_ALTERNATE) like '%" + email + "%') ";
			}

			// search names
			if (name != null && name.trim().length() != 0) {
				sql += "or (" +
					   "	upper(FIRST_NAME) like '%" + name + "%' or " +
					   "	upper(MIDDLE_NAME) like '%" + name + "%' or " +
					   "	upper(LAST_NAME) like '%" + name + "%') ";
			}
			
			// search phones
			if (phone != null && phone.trim().length() != 0) {
				sql += "or (" +
					   "	GSM_NO_PRIMARY like '%" + phone + "%' or " +
					   "	GSM_NO_ALTERNATE like '%" + phone + "%' or " +
					   "	HOME_PHONE like '%" + phone + "%' or " +
					   "	WORK_PHONE like '%" + phone + "%') ";
			}

			contacts = dao.readList(Contact.class, sql, new Object[] {username});

			String displayType = UserPrefsController.getUserSetting(auth, UserPrefConstants.displayType);
			if (displayType == null) {
				displayType = UserPrefConstants.DISPLAY_TYPE_NAME_FIRST;
			}

			contacts = org.claros.groupware.contacts.utility.Utility.sortContacts(contacts, displayType.equals(UserPrefConstants.DISPLAY_TYPE_NAME_FIRST));
		} finally {
			JdbcUtil.close(dao);
			dao = null;
		}
		return contacts;
	}

	/**
	 * @param auth
	 * @param address
	 * @param company
	 * @param email
	 * @param name
	 * @param phone
	 * @return
	 */
	public static List searchContactsByNameEmailNick(String username, String str, boolean onlyWithEmail) throws Exception {
		IGenericDao dao = null;
		List contacts = null;
		try {
			dao = Utility.getDbConnection();
			
			String sql = "select * from CONTACTS where user=? ";

			// search emails
			if (str != null && str.trim().length() != 0) {
				str = str.toUpperCase();

				sql += "and (" +
					   "	upper(EMAIL_PRIMARY) like '%" + str + "%' or " +
					   "	upper(EMAIL_ALTERNATE) like '%" + str + "%' or " +
					   "	upper(FIRST_NAME) like '%" + str + "%' or " +
					   "	upper(MIDDLE_NAME) like '%" + str + "%' or " +
					   "	upper(NICKNAME) like '%" + str + "%' or " +
					   "	upper(LAST_NAME) like '%" + str + "%' " + 
					   "	) ";
			}
			if (onlyWithEmail) {
				sql += "and EMAIL_PRIMARY like '%@%' ";
			}
			contacts = dao.readList(Contact.class, sql, new Object[] {username});
			contacts = org.claros.groupware.contacts.utility.Utility.sortContacts(contacts, true);
		} finally {
			JdbcUtil.close(dao);
			dao = null;
		}
		return contacts;
	}


	/**
	 * @param auth
	 * @param address
	 * @param company
	 * @param email
	 * @param name
	 * @param phone
	 * @return
	 */
	public static Contact searchContactByNick(String username, String str) throws Exception {
		IGenericDao dao = null;
		List contacts = null;
		Contact contact = null;
		try {
			dao = Utility.getDbConnection();
			String sql = "select * from CONTACTS where USER=? AND upper(NICKNAME) = ? ";
			contacts = dao.readList(Contact.class, sql, new Object[] {username, str.toUpperCase(new Locale("en", "US"))});
			if (contacts.size() == 1) {
				contact = (Contact)contacts.get(0);
			}
		} finally {
			JdbcUtil.close(dao);
			dao = null;
		}
		return contact;
	}


	public static Contact searchContactByEmail(String username, String email) throws Exception {
		IGenericDao dao = null;
		List contacts = null;
		Contact contact = null;
		try {
			dao = Utility.getDbConnection();
			String sql = "select * from CONTACTS where USER=? AND (" +
						 "	upper(EMAIL_PRIMARY) = ? or " +
						 "	upper(EMAIL_ALTERNATE) = ?)";
			email = email.toUpperCase(new Locale("en", "US"));
			contacts = dao.readList(Contact.class, sql, new Object[] {username, email, email});
			if (contacts.size() == 1) {
				contact = (Contact)contacts.get(0);
			}
		} finally {
			JdbcUtil.close(dao);
			dao = null;
		}
		return contact;
	}
}
