package org.claros.groupware.filters.controllers;

import java.util.List;

import org.claros.commons.exception.NoPermissionException;
import org.claros.commons.models.AuthProfile;
import org.claros.groupware.common.utility.Constants;
import org.claros.groupware.common.utility.Utility;
import org.claros.groupware.filters.models.Filter;

import com.jenkov.mrpersister.impl.mapping.AutoGeneratedColumnsMapper;
import com.jenkov.mrpersister.itf.IGenericDao;
import com.jenkov.mrpersister.itf.mapping.IObjectMappingKey;
import com.jenkov.mrpersister.util.JdbcUtil;

/**
 * @author Umut Gokbayrak
 */
public class FilterController {

	/**
	 * @param long1
	 */
	public static void deleteFilter(AuthProfile auth, Long filterId) throws Exception {
		Filter tmp = getFilterById(auth, filterId);
		if (!tmp.getUser().equals(auth.getUsername())) {
			throw new NoPermissionException();
		}

		IGenericDao dao = null;
		try {
			dao = Utility.getDbConnection();
			dao.deleteByPrimaryKey(Filter.class, filterId);
		} catch (Exception e) {
			// do nothing sier
		} finally {
			JdbcUtil.close(dao);
			dao = null;
		}
	}

	/**
	 * @param auth
	 * @return
	 */
	public static List getFilters(AuthProfile auth) throws Exception {
		IGenericDao dao = null;
		List filters = null;
		try {
			dao = Utility.getDbConnection();
			String username = auth.getUsername();
			
			String sql = "select * from FILTERS where user=?";
			filters = dao.readList(Filter.class, sql, new Object[] {username});
		} finally {
			JdbcUtil.close(dao);
			dao = null;
		}
		return filters;
	}

	/**
	 * @param filter
	 */
	public static void saveFilter(AuthProfile auth, Filter filter) throws Exception {
		IGenericDao dao = null;
		try {
			dao = Utility.getDbConnection();
			
			Long id = filter.getId();
			if (id == null) {
				// it is an insert
				IObjectMappingKey myObj = Constants.persistMan.getObjectMappingFactory().createInstance(Filter.class, new AutoGeneratedColumnsMapper(true));
				dao.insert(myObj, filter);
			} else {
				// it is an update
				Filter tmp = getFilterById(auth, filter.getId());
				if (!tmp.getUser().equals(auth.getUsername())) {
					throw new NoPermissionException();
				}
				dao.update(filter);
			}
		} finally {
			JdbcUtil.close(dao);
			dao = null;
		}
	}

	/**
	 * @param jid
	 * @return
	 */
	public static Filter getFilterById(AuthProfile auth, Long jid) throws Exception {
		IGenericDao dao = null;
		Filter result = null;
		try {
			dao = Utility.getDbConnection();
			result = (Filter)dao.readByPrimaryKey(Filter.class, jid);
			if (!result.getUser().equals(auth.getUsername())) {
				throw new NoPermissionException();
			}
		} finally {
			JdbcUtil.close(dao);
			dao = null;
		}
		return result;
	}
}
