package org.claros.groupware.notes.controllers;

import java.util.List;

import org.apache.commons.dbutils.QueryRunner;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.claros.commons.db.DbConfigList;
import org.claros.commons.exception.NoPermissionException;
import org.claros.commons.models.AuthProfile;
import org.claros.groupware.common.utility.Constants;
import org.claros.groupware.common.utility.Utility;
import org.claros.groupware.notes.models.NotesFolder;

import com.jenkov.mrpersister.impl.mapping.AutoGeneratedColumnsMapper;
import com.jenkov.mrpersister.itf.IGenericDao;
import com.jenkov.mrpersister.itf.mapping.IObjectMappingKey;
import com.jenkov.mrpersister.util.JdbcUtil;

/**
 * @author Umut Gokbayrak
 */
public class FolderController {
	private static Log log = LogFactory.getLog(FolderController.class);
	
	public static List getFolders(AuthProfile auth) throws Exception {
		IGenericDao dao = null;
		List folders = null;
		try {
			dao = Utility.getDbConnection();
			String username = auth.getUsername();
			
			String sql = "select * from NOTES_FOLDERS where user=?";

			folders = dao.readList(NotesFolder.class, sql, new Object[] {username});
		} finally {
			JdbcUtil.close(dao);
			dao = null;
		}
		return folders;
	}
	
	public static void saveFolder(AuthProfile auth, NotesFolder folder) throws Exception {
		IGenericDao dao = null;
		try {
			dao = Utility.getDbConnection();
			
			Long id = folder.getId();
			if (id == null) {
				// it is an insert
				IObjectMappingKey myObj = Constants.persistMan.getObjectMappingFactory().createInstance(NotesFolder.class, new AutoGeneratedColumnsMapper(true));
				dao.insert(myObj, folder);
			} else {
				// it is an update
				NotesFolder tmp = getFolderById(auth, folder.getId());
				String user = tmp.getUser();
				if (!auth.getUsername().equals(user)) {
					throw new NoPermissionException();
				}
				dao.update(folder);
			}
		} finally {
			JdbcUtil.close(dao);
			dao = null;
		}
	}

	public static void deleteFolder(AuthProfile auth, Long folderId) throws Exception {
		IGenericDao dao = null;
		try {
			dao = Utility.getTxnDbConnection();
			String username = auth.getUsername();

			NotesFolder folder = getFolderById(auth, folderId);
			if (!folder.getUser().equals(auth.getUsername())) {
				throw new NoPermissionException();
			}
			
			String sql = "delete from NOTES where user=? and folder_id = ?";
			// delete the notes under folder
			dao.executeUpdate(sql, new Object[] {username, folderId});
			// delete the folder
			dao.deleteByPrimaryKey(NotesFolder.class, folderId);
			dao.commit();
		} catch (Exception e) {
//			dao.rollback();
//			throw e;
			log.warn("error while deleting folder", e);
		} finally {
			JdbcUtil.close(dao);
			dao = null;
		}
	}

	/**
	 * @param auth
	 * @param folderId
	 * @return
	 */
	public static NotesFolder getFolderById(AuthProfile auth, Long folderId) throws Exception {
		IGenericDao dao = null;
		NotesFolder folder = null;
		try {
			dao = Utility.getDbConnection();
			String username = auth.getUsername();
			
			String sql = "select * from NOTES_FOLDERS where user=? and id = ?";

			folder = (NotesFolder)dao.read(NotesFolder.class, sql, new Object[] {username, folderId});
		} finally {
			JdbcUtil.close(dao);
			dao = null;
		}
		return folder;
	}

	public static void emptyFolder(AuthProfile auth, Long folderId) throws Exception {
		QueryRunner run = new QueryRunner(DbConfigList.getDataSourceById("file"));
		String username = auth.getUsername();
		String sql = "DELETE FROM NOTES where user=? and folder_id=?";
		run.update(sql, new Object[] {username, folderId});
	}

}
