package org.claros.groupware.webmail.actions;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Locale;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.codec.binary.Hex;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;

import org.claros.commons.exception.NoPermissionException;
import org.claros.commons.mail.models.Email;
import org.claros.commons.mail.models.EmailPart;
import org.claros.commons.mail.utility.Utility;
import org.claros.commons.models.AuthProfile;
import org.claros.groupware.common.actions.BaseLoggedAction;
import org.claros.groupware.contacts.controllers.ContactsController;
import org.claros.groupware.contacts.models.Contact;
import org.claros.groupware.preferences.controllers.UserPrefsController;
import org.claros.groupware.preferences.utility.UserPrefConstants;
import org.claros.groupware.webmail.models.ComposeFormBean;
import org.claros.groupware.webmail.utility.Constants;
import org.hsqldb.lib.MD5;

/**
 * @version 	1.0
 * @author
 */
public class ComposeEntryAction extends BaseLoggedAction {

	public ActionForward myExecute(ActionMapping mapping, ActionForm form, HttpServletRequest request, HttpServletResponse response) throws Exception {
		String action = (String)getVariable(request, "action");
		
		// learn if user wants HTML email or plain text email.
		AuthProfile auth = getAuthProfile(request);
		HashMap prefMap = UserPrefsController.getUserPreferences(auth);
		String mailFormat = (String)prefMap.get(UserPrefConstants.mailFormat);
		if (mailFormat == null || mailFormat.equals("html")) {
			request.setAttribute("HtmlMail", new Boolean(true));
		} else {
			request.setAttribute("HtmlMail", new Boolean(false));
		}
		
		// get the from address and prepare in a format which is to be shown.
		String fullName = (String)prefMap.get(UserPrefConstants.fullName);
		String from = (String)prefMap.get(UserPrefConstants.fromAddress);
		String fullFromAddress = fullName + " <" + from + ">";
		if (fullName == null || fullName.trim().length() == 0) {
			fullFromAddress = from;
		}
		request.setAttribute("FromAddress", fullFromAddress);

		String signature = (String)prefMap.get(UserPrefConstants.signature);
		String signaturePos = (String)prefMap.get(UserPrefConstants.signaturePos);
		if (signature != null) {
			if (signaturePos == null) {
				signaturePos = "top";
			}
		}
		
		// learn the type of this message. If it is a new mail, 
		// forward or something else. Each message type needs some predefined settings.
		if (action == null) {
			// do nothing just forward
		} else if (action.equals(Constants.NEW_MESSAGE)) {
			ComposeFormBean bean = new ComposeFormBean();
			bean.setMessageBody(addSignature(signature, signaturePos, mailFormat, ""));
			request.getSession().setAttribute("ComposeFormBean", bean);
			request.getSession().setAttribute("attachments", null);
		} else if (action.equals(Constants.FORWARD_MESSAGE)) {
			// delete the previoue values
			request.getSession().setAttribute("ComposeFormBean", null);
			request.getSession().setAttribute("attachments", null);

			// now create a new one.
			ComposeFormBean bean = new ComposeFormBean();
			Email email = (Email)request.getSession().getAttribute("email");
//			bean.setTo(email.getBaseHeader().getToShown());
//			bean.setCc(email.getBaseHeader().getCcShown());
			bean.setSubject("Fw: " + email.getBaseHeader().getSubject());
			
			String bd = email.getBodyText();
			if (mailFormat == null || mailFormat.equals("html")) {
				bd = "<blockquote class=\"claros_quote\" style=\"border-left: 1px solid rgb(204, 204, 204); margin: 0pt 0pt 0pt 0.8ex; padding-left: 1ex;\">" + bd + "</blockquote>";
				bd = "&nbsp;<br><br>On " + email.getBaseHeader().getDate().toString() + ", " + org.claros.commons.utility.Utility.htmlSpecialChars(email.getFrom()) + " wrote" + bd;
			} else {
				bd = "\n\n\nOn " + email.getBaseHeader().getDate().toString() + ", " + email.getFrom() + " wrote" + bd;
			}
			bean.setMessageBody(addSignature(signature, signaturePos, mailFormat, bd));

			request.getSession().setAttribute("ComposeFormBean", bean);
			request.getSession().setAttribute("attachments", organizeParts(email.getParts()));
		} else if (action.equals(Constants.REPLY_ALL_MESSAGE)) {
			// delete the previoue values
			request.getSession().setAttribute("ComposeFormBean", null);
			request.getSession().setAttribute("attachments", null);

			// now create a new one.
			ComposeFormBean bean = new ComposeFormBean();
			Email email = (Email)request.getSession().getAttribute("email");
			String to = email.getBaseHeader().getFromShown();
			if (email.getBaseHeader().getCc() != null && !email.getBaseHeader().getCc().equals("")) {
				to += ", " + email.getBaseHeader().getCcShown();
			}
			if (email.getBaseHeader().getToShown() != null && !email.getBaseHeader().getToShown().equals("")) {
				to += ", " + email.getBaseHeader().getToShown();
			}
			bean.setTo(to);
			
			bean.setSubject("Re: " + email.getBaseHeader().getSubject());

			String bd = email.getBodyText();
			if (mailFormat == null || mailFormat.equals("html")) {
				bd = "<blockquote class=\"claros_quote\" style=\"border-left: 1px solid rgb(204, 204, 204); margin: 0pt 0pt 0pt 0.8ex; padding-left: 1ex;\">" + bd + "</blockquote>";
				bd = "&nbsp;<br><br>On " + email.getBaseHeader().getDate().toString() + ", " + org.claros.commons.utility.Utility.htmlSpecialChars(email.getFrom()) + " wrote" + bd;
			} else {
				bd = "\n\n\nOn " + email.getBaseHeader().getDate().toString() + ", " + email.getFrom() + " wrote" + bd;
			}
			bean.setMessageBody(addSignature(signature, signaturePos, mailFormat, bd));

			request.getSession().setAttribute("ComposeFormBean", bean);
			request.getSession().setAttribute("attachments", null);
		} else if (action.equals(Constants.REPLY_MESSAGE)) {
			// delete the previoue values
			request.getSession().setAttribute("ComposeFormBean", null);
			request.getSession().setAttribute("attachments", null);

			// now create a new one.
			ComposeFormBean bean = new ComposeFormBean();
			Email email = (Email)request.getSession().getAttribute("email");
			String replyTo = Utility.addressArrToString(email.getBaseHeader().getReplyTo());
			if (replyTo == null || replyTo.trim().length() == 0) {
				bean.setTo(email.getBaseHeader().getFromShown());
			} else {
				bean.setTo(replyTo);
			}
			bean.setSubject("Re: " + email.getBaseHeader().getSubject());

			String bd = email.getBodyText();
			if (mailFormat == null || mailFormat.equals("html")) {
				bd = "<blockquote class=\"claros_quote\" style=\"border-left: 1px solid rgb(204, 204, 204); margin: 0pt 0pt 0pt 0.8ex; padding-left: 1ex;\">" + bd + "</blockquote>";
				bd = "&nbsp;<br><br>On " + email.getBaseHeader().getDate().toString() + ", " + org.claros.commons.utility.Utility.htmlSpecialChars(email.getFrom()) + " wrote" + bd;
			} else {
				bd = "\n\n\nOn " + email.getBaseHeader().getDate().toString() + ", " + email.getFrom() + " wrote" + bd;
			}
			bean.setMessageBody(addSignature(signature, signaturePos, mailFormat, bd));

			request.getSession().setAttribute("ComposeFormBean", bean);
			request.getSession().setAttribute("attachments", null);
		} else if (action.equals(Constants.CONTACT_MESSAGE)) {
			// a new message from contacts book.
			String sId = (String)getVariable(request, "id");
			Long id = new Long(Long.parseLong(sId));
			Contact contact = (Contact)ContactsController.getContactById(auth, id);
			if (!auth.getUsername().equals(contact.getUser())) {
				throw new NoPermissionException();
			}
				
			// delete the previous values
			request.getSession().setAttribute("ComposeFormBean", null);
			request.getSession().setAttribute("attachments", null);

			// now create a new one.
			ComposeFormBean bean = new ComposeFormBean();
			
			// get the contact's fullname and email address and generate the to field of the e-mail		
			String displayType = UserPrefsController.getUserSetting(auth, UserPrefConstants.displayType);
			boolean nameFirst = true;
			if (displayType != null && displayType.equals(UserPrefConstants.DISPLAY_TYPE_SURNAME_FIRST)) {
				nameFirst = false;
			}
			String fullAddress = "";
			if (nameFirst) {
				if (contact.getMiddleName() == null) {
					fullAddress = contact.getFirstName() + " " + contact.getLastName() + " <" + contact.getEmailPrimary() + ">";
				} else {
					fullAddress = contact.getFirstName() + " " + contact.getMiddleName() + " " + contact.getLastName() + " <" + contact.getEmailPrimary() + ">";
				}
			} else {
				if (contact.getMiddleName() == null) {
					fullAddress = contact.getLastName() + ", " + contact.getFirstName() + " <" + contact.getEmailPrimary() + ">";
				} else {
					fullAddress = contact.getLastName() + ", " + contact.getFirstName() + " " + contact.getMiddleName() + " <" + contact.getEmailPrimary() + ">";
				}
			}
			bean.setTo(fullAddress);
			bean.setMessageBody(addSignature(signature, signaturePos, mailFormat, ""));
			request.getSession().setAttribute("ComposeFormBean", bean);
		}

		byte b[] = request.getSession().getId().getBytes();
		request.setAttribute("sessionId", new String(Hex.encodeHex(MD5.digestBytes(b))).toUpperCase(new Locale("en", "US")));
		return mapping.findForward("/compose.jsp");
	}
	
	private ArrayList organizeParts(ArrayList parts) {
		if (parts != null) {
			EmailPart tmp = null;
			int pos = -1;
			for (int i=0; i<parts.size(); i++) {
				tmp = (EmailPart)parts.get(i);
				if (tmp.getContentType() == null) {
					tmp.setContentType("application/octet-stream");
				} else {
					pos = tmp.getContentType().indexOf(";");
					if (pos > 0) {
						tmp.setContentType(tmp.getContentType().substring(0, pos));
					}
				}
			}
		}
		return parts;
	}


	private String addSignature(String signature, String signaturePos, String mailFormat, String body) {
		if (mailFormat == null || mailFormat.equals("html")) {
			if (signaturePos.equals("top")) {
				body = signature + "<br>&nbsp;<br>" + body;
			} else {
				body = body + "<br>" + signature;
			}
		} else {
			if (signaturePos.equals("top")) {
				body = "\n\n" + signature + "\n" + body;
			} else {
				body = body + "\n" + signature + "\n";
			}
		}
		return body;
	}
}
