package org.claros.groupware.webmail.actions;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Locale;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;

import org.claros.commons.mail.comparator.EmailHeaderComparator;
import org.claros.commons.mail.models.ConnectionMetaHandler;
import org.claros.commons.mail.models.ConnectionProfile;
import org.claros.commons.models.AuthProfile;
import org.claros.groupware.common.actions.BaseLoggedAction;
import org.claros.groupware.webmail.controllers.FolderController;
import org.claros.groupware.webmail.controllers.InboxController;
import org.claros.groupware.webmail.factory.FolderControllerFactory;
import org.claros.groupware.webmail.factory.InboxControllerFactory;
import org.claros.groupware.webmail.models.FolderDbItem;
import org.claros.groupware.webmail.models.FolderDbItemWrapper;
import org.claros.groupware.webmail.utility.Constants;

/**
 * @version 	1.0
 * @author		Umut Gokbayrak
 */
public class MailboxAction extends BaseLoggedAction {
	private static Log log = LogFactory.getLog(MailboxAction.class);
	private static Locale loc = new Locale("en", "US");
	
	public ActionForward myExecute(ActionMapping mapping, ActionForm form, HttpServletRequest request, HttpServletResponse response) throws Exception {
		ActionForward forward = mapping.findForward("mailbox.jsp");
		
		AuthProfile auth = getAuthProfile(request);
		
		String sFolder = (String)getVariable(request, "fid");
		List headers = null;

		ConnectionMetaHandler handler = null;
		ConnectionProfile profile = getConnectionProfile(request);

		FolderControllerFactory foldFact = null;
		FolderController folderCont = null;
		String currFolder = org.claros.commons.mail.utility.Constants.FOLDER_INBOX;
		if (sFolder == null || sFolder.equals("") || sFolder.equals(currFolder)) {
			// if sFolder == currFOlder(which is INBOX at the moment) it is imap
			// and it is fetching the inbox. So do the mail filtering.
			try {
				InboxControllerFactory inFact = new InboxControllerFactory(auth, profile, handler);
				InboxController inCont = inFact.getInboxController();
				handler = inCont.checkEmail();
				request.getSession().setAttribute("handler", handler);
				foldFact = new FolderControllerFactory(auth, profile, handler);
				folderCont = foldFact.getFolderController();
			} catch (Exception e) {
				log.debug("error while checking new mail in mailboxaction", e);
			}

			if (profile.getProtocol().equals(org.claros.commons.mail.utility.Constants.POP3)) {
				currFolder = folderCont.getInboxFolder().getId().toString();
			} else {
				currFolder = folderCont.getInboxFolder().getFolderName();
			}
			// fetched some messages. It is probable that 
			// folder counters have changed.
			List folders = folderCont.getFolders();
			request.setAttribute("mailFolders", folders);
		} else {
			currFolder = sFolder;
			handler = getConnectionHandler(request);
			foldFact = new FolderControllerFactory(auth, profile, handler);
			folderCont = foldFact.getFolderController();
		}

		headers = folderCont.getHeadersByFolder(currFolder);
		request.getSession().setAttribute("fid", currFolder);
		
		if (headers != null) {
			Collections.sort(headers, new EmailHeaderComparator(false));
			request.setAttribute("totalMessages", Integer.toString(headers.size()));
		} else {
			request.setAttribute("totalMessages", Integer.toString(0));
		}
		request.setAttribute("headers", headers);

		// get current folder name
		FolderDbItem fItem = folderCont.getFolder(currFolder);
		request.setAttribute("folderName", fItem.getFolderName().toUpperCase(loc));
		if (fItem.getType().equals(Constants.FOLDER_TYPE_SENT)) {
			request.setAttribute("isSentItems", new Boolean(true));
		}

		// get all folders. It is already at the request. (set by BaseLoggedAction)
		List folders = (List)request.getAttribute("mailFolders");
		request.setAttribute("mailFolders", folders);

		// other folders (folders except current one)
		FolderDbItemWrapper tmp = null;
		ArrayList otherFolders = new ArrayList();
		for (int i=0;i<folders.size();i++) {
			tmp = (FolderDbItemWrapper)folders.get(i);

			if (profile.getProtocol().equals(org.claros.commons.mail.utility.Constants.POP3)) {
				if (!tmp.getId().equals(new Long(currFolder))) {
					otherFolders.add(tmp);
				}
			} else {
				if (!tmp.getFolderName().toUpperCase().equals(currFolder.toUpperCase())) {
					otherFolders.add(tmp);
				}
			}
		}
		request.setAttribute("otherFolders", otherFolders);
		
		return forward;
	}
}
