package org.claros.groupware.webmail.actions;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.PrintWriter;

import javax.servlet.ServletException;
import javax.servlet.ServletOutputStream;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.claros.commons.mail.models.Email;
import org.claros.commons.mail.models.EmailPart;
import org.claros.commons.mail.parser.HTMLMessageParser;
import org.claros.commons.utility.Utility;
import org.claros.groupware.common.actions.ClarosHttpServlet;

/**
 * @version 	1.0
 * @author		Umut Gokbayrak
 */
public class ShowPartServlet extends ClarosHttpServlet {

	/**
	* @see javax.servlet.http.HttpServlet#void (javax.servlet.http.HttpServletRequest, javax.servlet.http.HttpServletResponse)
	*/
	public void myGet(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
		int partId = 0;
		try {
			partId = Integer.parseInt((String)req.getAttribute("partid"));
		} catch (Exception e) {}

		Email email = (Email)req.getSession().getAttribute("email");
		EmailPart part = (EmailPart)email.getParts().get(partId);
		resp.setContentType(part.getContentType());
		resp.setHeader ("Pragma", "public");
		resp.setHeader ("Cache-Control", "must-revalidate");
		resp.setDateHeader ("Expires",0); 
		
		resp.setHeader("Content-disposition","inline; filename=" + part.getFileName());
//		resp.setHeader("Content-disposition","attachment; filename=" + part.getFileName());

		if (part.isPlainText()) {
			resp.setContentType("text/html; charset=UTF-8");
			PrintWriter out = resp.getWriter();
			String content = (String)part.getContent();
			content = Utility.replaceAllOccurances(content, "\n", "<br>");
			content = Utility.doCharsetCorrections(content);
			if (content.indexOf("") >= 0) {
				content = content.substring(0, content.indexOf(""));
			}
			Boolean showCss = (Boolean)req.getSession().getAttribute("showDefaultCss");
			if (showCss != null && showCss.booleanValue()) {
				content = "<style type=\"text/css\"> body {font-family: Verdana, Arial, Helvetica, sans-serif; font-size: 10pt}</style>" + content;
			}
			resp.setHeader("Content-length", Integer.toString(content.length()));
			out.write(content);
		} else if (part.isHTMLText()) {
			resp.setContentType("text/html; charset=UTF-8");
			PrintWriter out = resp.getWriter();
			String content = (String)part.getContent();
			content = HTMLMessageParser.prepareInlineHTMLContent(email, content);
			content = Utility.convertTRCharsToHtmlSafe(content);
			resp.setHeader("Content-length", Integer.toString(content.length()));
			out.write(content);
		} else {
			String tmpContType = (part.getContentType() == null) ? "application/octet-stream" : part.getContentType();
			int pos = tmpContType.indexOf(";");
			if (pos >= 0) {
				tmpContType = tmpContType.substring(0, pos);
			}
			resp.setContentType(tmpContType);
			
			Object obj = part.getContent();
			if (obj instanceof ByteArrayOutputStream) {
				ServletOutputStream sos = resp.getOutputStream();
				ByteArrayOutputStream baos = (ByteArrayOutputStream)obj;
				byte[] b = baos.toByteArray();
				resp.setHeader("Content-length",Integer.toString(b.length));
				sos.write(b);
//				sos.flush();
				sos.close();
			} else if (obj instanceof String) {
				PrintWriter out = resp.getWriter();
				String content = (String)part.getContent();
				resp.setHeader("Content-length", Integer.toString(content.length()));
				out.write(content);
			} else if (obj instanceof ByteArrayInputStream) {
				ServletOutputStream sos = resp.getOutputStream();
				ByteArrayInputStream bais = (ByteArrayInputStream)obj;
				int i = -1;
				int len = 0;
				while ((i = bais.read()) != -1) {
					sos.write(i);
					len++;
				}
				resp.setHeader("Content-length",Integer.toString(len));
				sos.close();
			}
		}
	}

	/* (non-Javadoc)
	 * @see org.claros.groupware.common.actions.ClarosHttpServlet#myPost(javax.servlet.http.HttpServletRequest, javax.servlet.http.HttpServletResponse)
	 */
	protected void myPost(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		myGet(request, response);
	}
}
