package org.claros.groupware.webmail.controllers;

import java.io.ByteArrayInputStream;
import java.sql.SQLException;
import java.util.Properties;

import javax.mail.Session;
import javax.mail.internet.MimeMessage;

import org.apache.commons.dbutils.QueryRunner;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.claros.commons.db.DbConfigList;
import org.claros.commons.exception.NoPermissionException;
import org.claros.commons.mail.exception.MailboxActionException;
import org.claros.commons.mail.models.ConnectionMetaHandler;
import org.claros.commons.mail.models.ConnectionProfile;
import org.claros.commons.mail.models.Email;
import org.claros.commons.mail.parser.MessageParser;
import org.claros.commons.models.AuthProfile;
import org.claros.groupware.common.utility.Constants;
import org.claros.groupware.common.utility.Utility;
import org.claros.groupware.webmail.models.EmailDbItem;

import com.jenkov.mrpersister.impl.mapping.AutoGeneratedColumnsMapper;
import com.jenkov.mrpersister.itf.IGenericDao;
import com.jenkov.mrpersister.itf.mapping.IObjectMappingKey;
import com.jenkov.mrpersister.util.JdbcUtil;
/**
 * @author Umut Gokbayrak
 */
public class DbMailControllerImpl implements MailController {
	private static Log log = LogFactory.getLog(DbMailControllerImpl.class);
	private AuthProfile auth;
	private ConnectionProfile profile;
	private ConnectionMetaHandler handler;
	
	private DbMailControllerImpl() {
		super();
	}
	
	public DbMailControllerImpl(AuthProfile auth, ConnectionProfile profile, ConnectionMetaHandler handler) {
		this.auth = auth;
		this.profile = profile;
		this.handler = handler; 
	}
 
	/* (non-Javadoc)
	 * @see org.claros.groupware.webmail.controllers.MailController#getEmailById(java.lang.Long)
	 */
	public Email getEmailById(Long emailId) throws Exception {
		Email email = null;
		EmailDbItem item = getEmailDbItemById(emailId);
		
		Properties props = new Properties();
		Session session = Session.getDefaultInstance(props);
		ByteArrayInputStream bis = new ByteArrayInputStream(item.getEmail());
		MimeMessage msg = new MimeMessage(session, bis);

		try {
			email = MessageParser.parseMessage(msg);
			email.setMsgId(item.getId());
		} catch (Exception e) {
			log.error("Message could not be parsed to an email object", e);
			throw new MailboxActionException(e);
		}
		if (email != null) {
			email.setMsgId(item.getId());
		}
		return email;
	}

	/**
	 * 
	 * @param auth
	 * @param emailId
	 * @return
	 * @throws Exception
	 */
	private EmailDbItem getEmailDbItemById(Long emailId) throws Exception {
		EmailDbItem email = null;
		IGenericDao dao = null;
		try {
			dao = Utility.getDbConnection();
			String username = auth.getUsername();
			
			String sql = "SELECT * FROM EMAIL_DB_ITEMS WHERE USER=? AND ID = ?";
			email = (EmailDbItem)dao.read(EmailDbItem.class, sql, new Object[] {username, emailId});
		} finally {
			JdbcUtil.close(dao);
			dao = null;
		}
		return email;
	}


	/* (non-Javadoc)
	 * @see org.claros.groupware.webmail.controllers.MailController#deleteEmail(java.lang.Long)
	 */
	public void deleteEmail(Long emailId) throws Exception {
		EmailDbItem tmp = getEmailDbItemById(emailId);
		if (tmp != null) {
			String user = tmp.getUser();
			if (!auth.getUsername().equals(user)) {
				throw new NoPermissionException();
			}
		
			IGenericDao dao = null;
			try {
				dao = Utility.getDbConnection();
				dao.deleteByPrimaryKey(EmailDbItem.class, emailId);
			} catch (Exception e) {
				// do nothing sier
			} finally {
				JdbcUtil.close(dao);
				dao = null;
			}
		}
	}

	/* (non-Javadoc)
	 * @see org.claros.groupware.webmail.controllers.MailController#moveEmail(java.lang.Long, java.lang.String)
	 */
	public void moveEmail(Long msgId, String destFolder) throws Exception {
		EmailDbItem item = getEmailDbItemById(msgId);
		Long destId = new Long(destFolder);
		item.setFolderId(destId);
		updateEmail(item);
	}

	/**
	 * 
	 * @param item
	 * @throws Exception
	 */
	private void updateEmail(EmailDbItem item) throws Exception {
		EmailDbItem tmp = getEmailDbItemById(item.getId());
		String user = tmp.getUser();
		if (!auth.getUsername().equals(user)) {
			throw new NoPermissionException();
		}

		IGenericDao dao = null;
		try {
			dao = Utility.getDbConnection();
			dao.update(EmailDbItem.class, item);
		} finally {
			JdbcUtil.close(dao);
			dao = null;
		}
	}

	/* (non-Javadoc)
	 * @see org.claros.groupware.webmail.controllers.MailController#appendEmail(org.claros.groupware.webmail.models.EmailDbItem)
	 */
	public void appendEmail(EmailDbItem item) throws Exception {
		IGenericDao dao = null;
		try {
			dao = Utility.getDbConnection();
		
			IObjectMappingKey myObj = Constants.persistMan.getObjectMappingFactory().createInstance(EmailDbItem.class, new AutoGeneratedColumnsMapper(true));
			dao.insert(myObj, item);
		} finally {
			JdbcUtil.close(dao);
			dao = null;
		}
	}

	/* (non-Javadoc)
	 * @see org.claros.groupware.webmail.controllers.MailController#markAsRead(java.lang.Long)
	 */
	public void markAsRead(Long msgId) throws Exception {
		QueryRunner run = new QueryRunner(DbConfigList.getDataSourceById("file"));
		String username = auth.getUsername();
		try {
			String sql = "UPDATE EMAIL_DB_ITEMS SET UNREAD = ? where user=? and id=?";
			run.update(sql, new Object[] {new Boolean(false), username, msgId});
		} catch (SQLException e) {
			throw e;
		}
	}

	/**
	 * 
	 * @param auth
	 * @param md5Header
	 * @return
	 * @throws Exception
	 */
	public boolean mailAlreadyFetched(String md5Header) throws Exception {
		IGenericDao dao = null;
		boolean result = false;
		try {
			dao = Utility.getDbConnection();
			String username = auth.getUsername();
			
			String sql = "SELECT * FROM EMAIL_DB_ITEMS WHERE USER=? AND UNIQUE_ID = ?";
			EmailDbItem email = (EmailDbItem)dao.read(EmailDbItem.class, sql, new Object[] {username, md5Header});
			if (email != null) {
				result = true;
			}
		} finally {
			JdbcUtil.close(dao);
			dao = null;
		}
		return result;
	}

	/* (non-Javadoc)
	 * @see org.claros.groupware.webmail.controllers.MailController#deleteEmails(int)
	 */
	public void deleteEmails(int msgs[]) throws Exception {
		for (int i=0; i<msgs.length; i++) {
			deleteEmail(new Long(msgs[i]));
		}
	}
}
