package org.claros.groupware.webmail.controllers;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

import javax.mail.Folder;
import javax.mail.MessagingException;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.claros.commons.exception.SystemException;
import org.claros.commons.mail.models.ConnectionMetaHandler;
import org.claros.commons.mail.models.ConnectionProfile;
import org.claros.commons.mail.protocols.ImapProtocolImpl;
import org.claros.commons.mail.protocols.Protocol;
import org.claros.commons.mail.protocols.ProtocolFactory;
import org.claros.commons.models.AuthProfile;
import org.claros.groupware.webmail.models.FolderDbItem;
import org.claros.groupware.webmail.models.FolderDbItemWrapper;
import org.claros.groupware.webmail.utility.Constants;

/**
 * @author Umut Gokbayrak
 */
public class ImapFolderControllerImpl implements FolderController {
	private AuthProfile auth;
	private ConnectionProfile profile;
	private ConnectionMetaHandler handler;
	private static Log log = LogFactory.getLog(ImapFolderControllerImpl.class);

	/**
	 * @param auth
	 * @param profile
	 * @param handler
	 */
	public ImapFolderControllerImpl(AuthProfile auth, ConnectionProfile profile, ConnectionMetaHandler handler) {
		this.auth = auth;
		this.profile = profile;
		this.handler = handler;
	}

	/**
	 * used to disable it.
	 */
	private ImapFolderControllerImpl() {
		super();
	}

	/* (non-Javadoc)
	 * @see org.claros.groupware.webmail.controllers.FolderController#getFolders()
	 */
	public List getFolders() throws Exception {
		ProtocolFactory factory = new ProtocolFactory(profile, auth, handler);
		ImapProtocolImpl protocol = (ImapProtocolImpl)factory.getImap(null);
		Folder folders[] = protocol.listFolders();
		
		ArrayList res = new ArrayList();
		boolean inboxAdded = false;
		if (folders != null) {
			Folder tmp = null;
			String n, fn = null;
			for (int i=0; i<folders.length; i++) {
				try {
					tmp = folders[i];
					if (tmp != null) {
						n = tmp.getName();
						fn = tmp.getFullName();
						if (fn != null && fn.startsWith("INBOX.")) {
						//	n = fn;
						}
						Integer type = determineFolderType(n);
						if (type.equals(Constants.FOLDER_TYPE_INBOX)) {
							inboxAdded = true;
						}
						FolderDbItem item = new FolderDbItem(null, null, auth.getUsername(), n, type);
						FolderDbItemWrapper wr = new FolderDbItemWrapper(item);
						wr.setUnreadItemCount(new Integer(tmp.getUnreadMessageCount()));
						wr.setTotalItemCount(new Integer(tmp.getMessageCount()));
						res.add(wr);
					}
				} catch (MessagingException e) {
					// do not worry about this. 
					log.debug(e);
				}
			}
		}
		
		// inbox is not added in this server implementation(weird!!!) . Please add it.
		if (!inboxAdded) {
			FolderDbItem item = new FolderDbItem(null, null, auth.getUsername(), org.claros.commons.mail.utility.Constants.FOLDER_INBOX.toUpperCase(), Constants.FOLDER_TYPE_INBOX);
			FolderDbItemWrapper wr = new FolderDbItemWrapper(item);
			Folder inbox = protocol.getFolder();
			if (inbox.exists()) {
				wr.setUnreadItemCount(new Integer(inbox.getUnreadMessageCount()));
				wr.setTotalItemCount(new Integer(inbox.getMessageCount()));
				res.add(wr);
			}
		}
		
		Collections.sort(res, new Comparator() {
			public int compare(Object f1, Object f2) {
				FolderDbItemWrapper fw1 = (FolderDbItemWrapper)f1;
				FolderDbItemWrapper fw2 = (FolderDbItemWrapper)f2;
				
				Integer t1 = fw1.getType(); 
				Integer t2 = fw2.getType();
				
				if (t1.equals(Constants.FOLDER_TYPE_CUSTOM) && t2.equals(Constants.FOLDER_TYPE_CUSTOM)) {
					return fw1.getFolderName().compareTo(fw2.getFolderName());
				} else {
					return t1.compareTo(t2);
				}
			}
		});

		return res;
	}

	/**
	 * 
	 * @param folderName
	 * @return
	 */
	private Integer determineFolderType(String folderName) {
		if (folderName == null) {
			return null;
		}
		if (folderName.toUpperCase().equals(org.claros.commons.mail.utility.Constants.FOLDER_INBOX.toUpperCase())) {
			return Constants.FOLDER_TYPE_INBOX;
		} else if (folderName.toUpperCase().equals(org.claros.commons.mail.utility.Constants.FOLDER_SENT.toUpperCase())) {
			return Constants.FOLDER_TYPE_SENT;
		} else if (folderName.toUpperCase().equals(org.claros.commons.mail.utility.Constants.FOLDER_JUNK.toUpperCase())) {
			return Constants.FOLDER_TYPE_JUNK;
		} else {
			return Constants.FOLDER_TYPE_CUSTOM;
		}
	}

	/* (non-Javadoc)
	 * @see org.claros.groupware.webmail.controllers.FolderController#getFolder(java.lang.String)
	 */
	public FolderDbItem getFolder(String folder) throws Exception {
		ProtocolFactory factory = new ProtocolFactory(profile, auth, handler);
		ImapProtocolImpl protocol = (ImapProtocolImpl)factory.getImap(folder);
		Folder f = protocol.getFolder();

		FolderDbItem item = null;
		if (f.exists()) {
			item = new FolderDbItem(null, null, auth.getUsername(), f.getName(), determineFolderType(f.getName()));
		}
		ImapProtocolImpl.closeFolder(f);
		return item;
	}

	/* (non-Javadoc)
	 * @see org.claros.groupware.webmail.controllers.FolderController#getMailsByFolder(java.lang.String)
	 */
	public List getMailsByFolder(String folder) throws Exception {
		throw new SystemException("Method not supported at this protocol");
	}

	/* (non-Javadoc)
	 * @see org.claros.groupware.webmail.controllers.FolderController#createFolder(org.claros.groupware.webmail.models.FolderDbItem)
	 */
	public void createFolder(FolderDbItem item) throws Exception {
		String name = item.getFolderName();
		ProtocolFactory factory = new ProtocolFactory(profile, auth, handler);
		ImapProtocolImpl protocol = (ImapProtocolImpl)factory.getImap(name);
		protocol.createFolder();
	}

	/* (non-Javadoc)
	 * @see org.claros.groupware.webmail.controllers.FolderController#countUnreadMessages(java.lang.String)
	 */
	public Integer countUnreadMessages(String folder) throws Exception {
		ProtocolFactory factory = new ProtocolFactory(profile, auth, handler);
		ImapProtocolImpl protocol = (ImapProtocolImpl)factory.getImap(folder);
		int count = protocol.getUnreadMessageCount();
		return new Integer(count);
	}
	
	/* (non-Javadoc)
	 * @see org.claros.groupware.webmail.controllers.FolderController#countTotalMessages(java.lang.String)
	 */
	public Integer countTotalMessages(String folder) throws Exception {
		ProtocolFactory factory = new ProtocolFactory(profile, auth, handler);
		ImapProtocolImpl protocol = (ImapProtocolImpl)factory.getImap(folder);
		int count = protocol.getTotalMessageCount();
		return new Integer(count);
	}

	/* (non-Javadoc)
	 * @see org.claros.groupware.webmail.controllers.FolderController#emptyFolder(java.lang.String)
	 */
	public void emptyFolder(String folder) throws Exception {
		ProtocolFactory factory = new ProtocolFactory(profile, auth, handler);
		ImapProtocolImpl protocol = (ImapProtocolImpl)factory.getImap(folder);
		protocol.emptyFolder();
	}

	/* (non-Javadoc)
	 * @see org.claros.groupware.webmail.controllers.FolderController#deleteFolder(java.lang.String)
	 */
	public void deleteFolder(String folder) throws Exception {
		ProtocolFactory factory = new ProtocolFactory(profile, auth, handler);
		ImapProtocolImpl protocol = (ImapProtocolImpl)factory.getImap(folder);
		protocol.deleteFolder();
	}

	/* (non-Javadoc)
	 * @see org.claros.groupware.webmail.controllers.FolderController#renameFolder(java.lang.String, java.lang.String)
	 */
	public void renameFolder(String oldName, String newName) throws Exception {
		ProtocolFactory factory = new ProtocolFactory(profile, auth, handler);
		ImapProtocolImpl protocol = (ImapProtocolImpl)factory.getImap(oldName);
		protocol.renameFolder(newName);
	}

	/* (non-Javadoc)
	 * @see org.claros.groupware.webmail.controllers.FolderController#getHeadersByFolder(java.lang.String)
	 */
	public ArrayList getHeadersByFolder(String folder) throws Exception {
		ProtocolFactory factory = new ProtocolFactory(profile, auth, handler);
		Protocol protocol = factory.getProtocol(folder);
		ArrayList headers = protocol.fetchAllHeaders();
		
		return headers;
	}

	/* (non-Javadoc)
	 * @see org.claros.groupware.webmail.controllers.FolderController#createDefaultFolders()
	 */
	public void createDefaultFolders() throws Exception {		
		ProtocolFactory factory = new ProtocolFactory(profile, auth, handler);

		ImapProtocolImpl imapJunk = (ImapProtocolImpl)factory.getImap(org.claros.commons.mail.utility.Constants.FOLDER_JUNK);
		imapJunk.createFolder();
		
		ImapProtocolImpl imapSent = (ImapProtocolImpl)factory.getImap(org.claros.commons.mail.utility.Constants.FOLDER_SENT);
		imapSent.createFolder();
	}

	/**
	 * @return
	 * @throws Exception
	 */
	public FolderDbItem getJunkFolder() throws Exception {
		return new FolderDbItem(null, null, null, org.claros.commons.mail.utility.Constants.FOLDER_JUNK, null);
	}

	/**
	 * @param auth
	 * @return
	 */
	public FolderDbItem getInboxFolder() throws Exception {
		return new FolderDbItem(null, null, null, org.claros.commons.mail.utility.Constants.FOLDER_INBOX, null);
	}

	/**
	 * @param auth
	 * @return
	 */
	public FolderDbItem getSentItems() throws Exception {
		return new FolderDbItem(null, null, null, org.claros.commons.mail.utility.Constants.FOLDER_SENT, null);
	}
}
