package org.claros.taglibs.page;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;

import org.apache.commons.codec.binary.Hex;
import org.hsqldb.lib.MD5;

/**
 * @author Umut Gokbayrak
 */
public class Paginator {
	private static HashMap paginatorData = new HashMap();

	public static PaginatorHandler paginateData(Collection col, int itemsPerPage) {
		if (col == null || col.size() == 0) {
			return null;
		}

		// ilk i handler' hazrlyoruz. Daha sonra dataya bu handler sayesinde
		// ulaacaz. Ayrca bu handler data hakknda baz basic bilgileri ierir.
		int size = col.size();
		
		int totalPages = (int)Math.ceil((double)size / (double)itemsPerPage);
		PaginatorHandler handler = new PaginatorHandler();
		handler.setItemsPerPage(itemsPerPage);
		handler.setId(getUniqueId());
		handler.setTotalPages(totalPages);
		handler.setTotalItems(size);

		Object obj = null;
		int pageCount = 0;

		ArrayList pages = new ArrayList();
		PageData page = new PageData();
		Iterator iter = col.iterator();
		
		int allCounter = 0;
		boolean justAdded = false;
		while (iter.hasNext()) {
			justAdded = false;
			obj = iter.next();
			page.addData(obj);
			page.setPageId(allCounter);
			page.setPageTitle("" + (allCounter + 1));
			
			if (allCounter == 0) {
				page.setFirstPage(true);
			}
			pageCount++; allCounter++;
			if (size == allCounter) {
				page.setLastPage(true);
			}
			if (pageCount == itemsPerPage) {
				pageCount = 0;
				pages.add(page);
				page = new PageData();
				justAdded = true;
			}
		}
		if (!justAdded && allCounter != 0) {
			pages.add(page);
		}
		
		paginatorData.put(handler.getId(), pages);
		return handler;
	}

	public static PageData getPage(String handlerId, int pageNo) {
		ArrayList pages = (ArrayList)paginatorData.get(handlerId);
		if (pages != null) {
			return (PageData)pages.get(pageNo);
		}
		return null;
	}
	
	public static int getPageCount(String handlerId) {
		ArrayList pages = (ArrayList)paginatorData.get(handlerId);
		if (pages != null) {
			return pages.size();
		}
		return 0;
	}

	/**
	 * Paginator class iinde bir memory leak yaratmamak iin kullancnn session' 
	 * destroy olduunda bellekteki paginator datas da silinmelidir. web.xml'de tanmlanan 
	 * bir SessionListener listener sayesinde session destroy olduunda bu metod 
	 * otomatikman arlr. Biz de temizliimizi yaparz.  
	 * @param id
	 */
	public static void invalidateSession(String id) {
		if (id != null) {
			paginatorData.remove(id);
		}
	}
	
	public static boolean cacheExists(String paginatorId) {
		return paginatorData.get(paginatorId) != null;
	}

	private static String getUniqueId() {
		double id = Math.random();
		return new String(Hex.encodeHex(MD5.digestBytes(("" + id).getBytes()))).toUpperCase();
	}

	/*
	public static ArrayList generatePageIndexes(String paginatorId, int currentPageId, int itemsPerPage) {
		int pageIndexes[] = new int[itemsPerPage];
		int tmp = currentPageId + (itemsPerPage - PaginatorConstants.centerPage);
		
		int size = getPageCount(paginatorId);
		if (tmp < itemsPerPage - 1) {
			tmp = itemsPerPage - 1;
		}
		if (tmp >= size) {
			tmp = size - 1;
		}
		for (int i = itemsPerPage - 1; i >= 0; i--) {
			pageIndexes[i] = tmp + 1;
			tmp--;
			if (tmp < 0) {
				break;
			}
		}
		ArrayList out = new ArrayList();
		for (int i=0;i<pageIndexes.length;i++) {
			if (pageIndexes[i] != 0) {
				out.add(new Integer(pageIndexes[i]));
			} 
		}
		return out;
	}
	*/
}
